(ns com.aurellem.vba
  (:import java.io.File)
  (:import org.apache.commons.io.FileUtils))

(defn vba-bytes [#^File vba]
  (let [bytes (FileUtils/readFileToByteArray vba)
        ints (int-array (count bytes))]
    (areduce bytes idx _ nil
             (aset ints idx
                   (bit-and 0xFF (aget bytes idx))))
    ints))

(def button-mask
  {;; main buttons
   :a         0x0001
   :b         0x0002

   ;; directional pad
   :r         0x0010
   :l         0x0020
   :u         0x0040
   :d         0x0080

   ;; meta buttons
   :select    0x0004
   :start     0x0008

   ;; hard reset -- not really a button
   :reset   0x0800})

(defn button-code [buttons]
  (reduce bit-or 0x0000 (map button-mask buttons)))

(defn buttons [mask]
  (loop [buttons []
         masks (seq button-mask)]
    (if (empty? masks) buttons
        (let [[button value] (first masks)]
          (if (not= 0x0 (bit-and value mask))
            (recur (conj buttons button) (rest masks))
            (recur buttons (rest masks)))))))

(def vba-header-length 257)

(defn vba-masks [#^File vba]
  (map (fn [[a b]]
         (+ (bit-shift-left a 8) b))
       (partition
        2 (drop vba-header-length (vba-bytes vba)))))

(defn vba-buttons [#^File vba]
  (map buttons (vba-masks vba)))
  


         
    
  